@@ -66,6 +66,9 @@
    returns a byte from any location in the file.
    Returns '\n' if out of bounds.
  */
+
+#ifndef NO_INLINE_GETBYTE
+
 int edit_get_byte (WEdit * edit, long byte_index)
 {
     unsigned long p;
@@ -80,6 +83,8 @@ int edit_get_byte (WEdit * edit, long byte_index)
     }
 }
 
+#endif
+
 char *edit_get_buffer_as_text (WEdit * e)
 {
     int l, i;
@@ -110,7 +115,7 @@ int init_dynamic_edit_buffers (WEdit * edit, const char *filename, const char *t
     if (filename)
 	if ((file = open ((char *) filename, O_RDONLY | MY_O_TEXT)) == -1) {
 /* The file-name is printed after the ':' */
-	    edit_error_dialog (_(" Error "), get_sys_error (catstrs (_(" Fail trying to open file for reading: "), filename, " ", 0)));
+	    edit_error_dialog (_(" Error "), get_sys_error (catstrs (_(" Failed trying to open file for reading: "), filename, " ", 0)));
 	    return 1;
 	}
     edit->curs2 = edit->last_byte;
@@ -153,7 +158,7 @@ int edit_load_file (WEdit * edit, const char *filename, const char *text, unsign
 	edit->last_byte = text_size;
 	filename = NULL;
     } else {
-#ifdef MIDNIGHT
+#if defined(MIDNIGHT) || defined(GTK)
 	if ((file = open ((char *) filename, O_RDONLY)) < 0)
 	{
 		close(creat((char *) filename, 0666));
@@ -165,7 +170,7 @@ int edit_load_file (WEdit * edit, const char *filename, const char *text, unsign
 	}
 #else
 	if ((file = open ((char *) filename, O_RDONLY)) < 0) {
-	    edit_error_dialog (_(" Error "), get_sys_error (catstrs (_(" Fail trying to open file for reading: "), filename, " ", 0)));
+	    edit_error_dialog (_(" Error "), get_sys_error (catstrs (_(" Failed trying to open file for reading: "), filename, " ", 0)));
 	    return 1;
 	}
 #endif
@@ -211,18 +216,22 @@ void edit_set_space_width (int s)
 
 #endif
 
-/* fills in the edit struct. returns 0 on fail. Pass edit as NULL for this function to do an malloc for you */
+/* fills in the edit struct. returns 0 on fail. Pass edit as NULL for this */
 WEdit *edit_init (WEdit * edit, int lines, int columns, const char *filename, const char *text, const char *dir, unsigned long text_size)
 {
     char *f;
+    int to_free = 0;
 #ifndef MIDNIGHT
     if (option_long_whitespace)
 	edit_set_space_width (per_char[' '] * 2);
     else
 	edit_set_space_width (per_char[' ']);
 #endif
-    if (!edit)
+    if (!edit) {
 	edit = malloc (sizeof (WEdit));
+	memset (edit, 0, sizeof (WEdit));
+	to_free = 1;
+    }
     if (!edit) {
 	edit_error_dialog (_(" Error "), _(" Error allocating memory "));
 	return 0;
@@ -244,7 +253,8 @@ WEdit *edit_init (WEdit * edit, int lines, int columns, const char *filename, co
 	f = catstrs (dir, filename, 0);
     if (edit_load_file (edit, f, text, text_size)) {
 /* edit_load_file already gives an error message */
-	free (edit);
+	if (to_free)
+	    free (edit);
 	return 0;
     }
     edit->force |= REDRAW_PAGE;
@@ -260,7 +270,8 @@ WEdit *edit_init (WEdit * edit, int lines, int columns, const char *filename, co
     edit->undo_stack = malloc ((edit->stack_size + 10) * sizeof (long));
     if (!edit->undo_stack) {
 	edit_error_dialog (_(" Error "), _(" Error allocating memory "));
-	free (edit);
+	if (to_free)
+	    free (edit);
 	return 0;
     }
     edit->total_lines = edit_count_lines (edit, 0, edit->last_byte);
@@ -318,7 +329,7 @@ int edit_renew (WEdit * edit)
     return 1;
 }
 
-/* returns 1 on success */
+/* returns 1 on success, if returns 0, the edit struct would have been free'd */
 int edit_reload (WEdit * edit, const char *filename, const char *text, const char *dir, unsigned long text_size)
 {
     int lines = edit->num_widget_lines;
@@ -1024,12 +1035,13 @@ static int is_in_indent (WEdit *edit)
 
 static int left_of_four_spaces (WEdit *edit);
 
-static void edit_move_to_prev_col (WEdit * edit, long p)
+void edit_move_to_prev_col (WEdit * edit, long p)
 {
     edit_cursor_move (edit, edit_move_forward3 (edit, p, edit->prev_col, 0) - edit->curs1);
 
     if (is_in_indent (edit) && option_fake_half_tabs) {
 	edit_update_curs_col (edit);
+	if (space_width)
 	if (edit->curs_col % (HALF_TAB_SIZE * space_width)) {
 	    int q = edit->curs_col;
 	    edit->curs_col -= (edit->curs_col % (HALF_TAB_SIZE * space_width));
@@ -1387,6 +1399,7 @@ static void edit_left_delete_word (WEdit * edit)
     } while (my_type_of (c) == my_type_of (edit_get_byte (edit, edit->curs1 - 1)));
 }
 
+extern int column_highlighting;
 
 /*
    the start column position is not recorded, and hence does not
@@ -1415,6 +1428,12 @@ void edit_do_undo (WEdit * edit)
 	case DELETE:
 	    edit_delete (edit);
 	    break;
+	case COLUMN_ON:
+	    column_highlighting = 1;
+	    break;
+	case COLUMN_OFF:
+	    column_highlighting = 0;
+	    break;
 	}
 	if (ac >= 256 && ac < 512)
 	    edit_insert_ahead (edit, ac - 256);
@@ -1423,8 +1442,10 @@ void edit_do_undo (WEdit * edit)
 
 	if (ac >= MARK_1 - 2 && ac < MARK_2 - 2) {
 	    edit->mark1 = ac - MARK_1;
+	    edit->column1 = edit_move_forward3 (edit, edit_bol (edit, edit->mark1), 0, edit->mark1);
 	} else if (ac >= MARK_2 - 2 && ac < KEY_PRESS) {
 	    edit->mark2 = ac - MARK_2;
+	    edit->column2 = edit_move_forward3 (edit, edit_bol (edit, edit->mark2), 0, edit->mark2);
 	}
 	if (count++)
 	    edit->force |= REDRAW_PAGE;		/* more than one pop usually means something big */
@@ -1438,9 +1459,9 @@ void edit_do_undo (WEdit * edit)
 	edit->force |= REDRAW_PAGE;
     }
     edit->start_display = ac - KEY_PRESS;	/* see push and pop above */
-    edit_update_curs_row(edit);
+    edit_update_curs_row (edit);
 
-    done_undo:;
+  done_undo:;
     push_action_disabled = 0;
 }
 
@@ -1466,7 +1487,7 @@ static void edit_delete_to_line_begin (WEdit * edit)
     }
 }
 
-static void edit_delete_line (WEdit * edit)
+void edit_delete_line (WEdit * edit)
 {
     int c;
     do {
@@ -1724,6 +1745,14 @@ int edit_execute_key_command (WEdit * edit, int command, int char_for_insertion)
 	edit_push_key_press (edit);
 
     r = edit_execute_cmd (edit, command, char_for_insertion);
+#ifdef GTK
+    if (edit->stopped && edit->widget->destroy_me) {
+	(*edit->widget->destroy_me) (edit->widget->destroy_me_user_data);
+	return 0;
+    }
+#endif
+    if (column_highlighting)
+	edit->force |= REDRAW_PAGE;
 
     return r;
 }
@@ -1753,11 +1782,12 @@ int edit_execute_cmd (WEdit * edit, int command, int char_for_insertion)
 {
     int result = 1;
     edit->force |= REDRAW_LINE;
-    if (edit->found_len)
+    if (edit->found_len || column_highlighting)
 /* the next key press will unhighlight the found string, so update whole page */
 	edit->force |= REDRAW_PAGE;
 
     if (command / 100 == 6) {	/* a highlight command like shift-arrow */
+	column_highlighting = 0;
 	if (!edit->highlight || (edit->mark2 != -1 && edit->mark1 != edit->mark2)) {
 	    edit_mark_cmd (edit, 1);	/* clear */
 	    edit_mark_cmd (edit, 0);	/* marking on */
@@ -1971,14 +2001,32 @@ int edit_execute_cmd (WEdit * edit, int command, int char_for_insertion)
     case CK_Toggle_Insert:
 	edit->overwrite = (edit->overwrite == 0);
 #ifndef MIDNIGHT
+#ifdef GTK
+/* *** */
+#else
 	CSetCursorColor (edit->overwrite ? color_palette (24) : color_palette (19));
+#endif
 #endif
 	break;
 
     case CK_Mark:
+	if (edit->mark2 >= 0) {
+	    if (column_highlighting)
+		edit_push_action (edit, COLUMN_ON);
+	    column_highlighting = 0;
+	}
+	edit_mark_cmd (edit, 0);
+	break;
+    case CK_Column_Mark:
+	if (!column_highlighting)
+	    edit_push_action (edit, COLUMN_OFF);
+	column_highlighting = 1;
 	edit_mark_cmd (edit, 0);
 	break;
     case CK_Unmark:
+	if (column_highlighting)
+	    edit_push_action (edit, COLUMN_ON);
+	column_highlighting = 0;
 	edit_mark_cmd (edit, 1);
 	break;
 
@@ -2016,6 +2064,7 @@ int edit_execute_cmd (WEdit * edit, int command, int char_for_insertion)
 
     case CK_Save_As:
 #ifndef MIDNIGHT
+/*	if (COptionsOf (edit->widget) & EDITOR_NO_FILE) */
 	if (edit->widget->options & EDITOR_NO_FILE)
 	    break;
 #endif
@@ -2023,14 +2072,14 @@ int edit_execute_cmd (WEdit * edit, int command, int char_for_insertion)
 	break;
     case CK_Save:
 #ifndef MIDNIGHT
-	if (edit->widget->options & EDITOR_NO_FILE)
+	if (COptionsOf (edit->widget) & EDITOR_NO_FILE)
 	    break;
 #endif
 	edit_save_confirm_cmd (edit);
 	break;
     case CK_Load:
 #ifndef MIDNIGHT
-	if (edit->widget->options & EDITOR_NO_FILE)
+	if (COptionsOf (edit->widget) & EDITOR_NO_FILE)
 	    break;
 #endif
 	edit_load_cmd (edit);
@@ -2106,12 +2155,22 @@ int edit_execute_cmd (WEdit * edit, int command, int char_for_insertion)
     case CK_Save_Desktop:
     case CK_New_Window:
     case CK_Cycle:
-    case CK_Menu:
     case CK_Save_And_Quit:
     case CK_Check_Save_And_Quit:
     case CK_Run_Another:
 	result = 0;
 	break;
+    case CK_Menu:
+#ifdef GTK
+	if (edit->widget->menubar)
+	    gtk_menu_popup (GTK_MENU(
+		(GTK_MENU_ITEM (g_list_nth_data (GTK_MENU_BAR (edit->widget->menubar)->menu_shell.children, 0)))->submenu
+	    ), 0, 0, 0, 0, 1, 0);
+	result = 1;
+#else
+	result = 0;
+#endif
+	break;
     }
 
 #ifdef MIDNIGHT
