@@ -14,7 +14,7 @@
 #    	 include <unistd.h>
 #    endif
 #    include <string.h>
-#    include "../src/tty.h"
+#    include "src/tty.h"
 #    include <sys/stat.h>
 #    include <errno.h>
      
@@ -36,7 +36,11 @@
 #    	     include <unistd.h>
 #    	 endif
      
+#ifdef GTK
+#    include <string.h>
+#else
 #    include <my_string.h>
+#endif
 #    include <sys/stat.h>
      
 #    ifdef HAVE_FCNTL_H
@@ -68,28 +72,40 @@
 #    include <X11/Xutil.h>
 #    include <X11/Xresource.h>
 #    include "lkeysym.h"
+#ifndef GTK
 #    include "coolwidget.h"
 #    include "app_glob.c"
 #    include "coollocal.h"
 #    include "stringtools.h"
+#else
+#    include "gtk/gtk.h"
+#    include "gdk/gdkprivate.h"
+#    include "gdk/gdk.h"
+#    include "gtkedit.h"
+#    include "editcmddef.h"
+#    ifndef _
+#        define _(x) x
+#        define N_(x) x
+#    endif
+#endif
 
 #else
 
-#    include "../src/main.h"		/* for char *shell */
-#    include "../src/mad.h"
-#    include "../src/dlg.h"
-#    include "../src/widget.h"
-#    include "../src/color.h"
-#    include "../src/dialog.h"
-#    include "../src/mouse.h"
-#    include "../src/global.h"
-#    include "../src/help.h"
-#    include "../src/key.h"
-#    include "../src/wtools.h"		/* for QuickWidgets */
-#    include "../src/win.h"
-#    include "../vfs/vfs.h"
-#    include "../src/menu.h"
-#    include "../src/regex.h"
+#    include "src/main.h"		/* for char *shell */
+#    include "src/mad.h"
+#    include "src/dlg.h"
+#    include "src/widget.h"
+#    include "src/color.h"
+#    include "src/dialog.h"
+#    include "src/mouse.h"
+#    include "src/global.h"
+#    include "src/help.h"
+#    include "src/key.h"
+#    include "src/wtools.h"		/* for QuickWidgets */
+#    include "src/win.h"
+#    include "vfs/vfs.h"
+#    include "src/menu.h"
+#    include "src/regex.h"
 #    define WANT_WIDGETS
      
 #    define WIDGET_COMMAND (WIDGET_USER + 10)
@@ -97,6 +113,37 @@
 
 #endif
 
+#ifdef GTK
+/* unistd.h defines _POSIX_VERSION on POSIX.1 systems. */
+#if defined(HAVE_DIRENT_H) || defined(_POSIX_VERSION)
+#   include <dirent.h>
+#   define NLENGTH(dirent) (strlen ((dirent)->d_name))
+#else
+#   define dirent direct
+#   define NLENGTH(dirent) ((dirent)->d_namlen)
+
+#   ifdef HAVE_SYS_NDIR_H
+#       include <sys/ndir.h>
+#   endif /* HAVE_SYS_NDIR_H */
+
+#   ifdef HAVE_SYS_DIR_H
+#       include <sys/dir.h>
+#   endif /* HAVE_SYS_DIR_H */
+
+#   ifdef HAVE_NDIR_H
+#       include <ndir.h>
+#   endif /* HAVE_NDIR_H */
+#endif /* not (HAVE_DIRENT_H or _POSIX_VERSION) */
+#   ifndef _
+#      define _(x) x
+#      define N_(x) x
+#   endif
+#include "vfs/vfs.h"
+#    define CDisplay gdk_display
+#    define CRoot gdk_root_parent
+#    define Window GtkEdit *
+#endif
+
 #define SEARCH_DIALOG_OPTION_NO_SCANF	1
 #define SEARCH_DIALOG_OPTION_NO_REGEX	2
 #define SEARCH_DIALOG_OPTION_NO_CASE	4
@@ -134,8 +181,13 @@
 #define MOD_INVERSE		(1 << 7)
 
 #ifndef MIDNIGHT
-#    define EDIT_TEXT_HORIZONTAL_OFFSET 4
-#    define EDIT_TEXT_VERTICAL_OFFSET 3
+#    ifdef GTK
+#        define EDIT_TEXT_HORIZONTAL_OFFSET 0
+#        define EDIT_TEXT_VERTICAL_OFFSET 0
+#    else
+#        define EDIT_TEXT_HORIZONTAL_OFFSET 4
+#        define EDIT_TEXT_VERTICAL_OFFSET 3
+#    endif
 #else
 #    define EDIT_TEXT_HORIZONTAL_OFFSET 0
 #    define EDIT_TEXT_VERTICAL_OFFSET 1
@@ -153,14 +205,14 @@
 /*there are a maximum of ... */
 #define MAXBUFF 1024
 /*... edit buffers, each of which is ... */
-#define EDIT_BUF_SIZE 16384
+#define EDIT_BUF_SIZE 0x10000
 /* ...bytes in size. */
 
 /*x / EDIT_BUF_SIZE equals x >> ... */
-#define S_EDIT_BUF_SIZE 14
+#define S_EDIT_BUF_SIZE 16
 
 /* x % EDIT_BUF_SIZE is equal to x && ... */
-#define M_EDIT_BUF_SIZE 16383
+#define M_EDIT_BUF_SIZE 0xFFFF
 
 #define SIZE_LIMIT (EDIT_BUF_SIZE * (MAXBUFF - 2))
 /* Note a 16k stack is 64k of data and enough to hold (usually) around 10
@@ -178,6 +230,8 @@
 #define STACK_BOTTOM 605
 #define CURS_LEFT_LOTS 606
 #define CURS_RIGHT_LOTS 607
+#define COLUMN_ON 608
+#define COLUMN_OFF 609
 #define MARK_1 1000
 #define MARK_2 700000000
 #define KEY_PRESS 1400000000
@@ -242,6 +296,8 @@ struct context_rule {
 struct editor_widget {
 #ifdef MIDNIGHT
     Widget widget;
+#elif defined(GTK)
+    GtkEdit *widget;
 #else
     struct cool_widget *widget;
 #endif
@@ -280,7 +336,7 @@ struct editor_widget {
     unsigned char overwrite;
     unsigned char modified;	/*has the file been changed?: 1 if char inserted or
 				   deleted at all since last load or save */
-#ifdef MIDNIGHT
+#if defined(MIDNIGHT) || defined(GTK)
     int delete_file;			/* has the file been created in edit_load_file? Delete
 			           it at end of editing when it hasn't been modified 
 				   or saved */
@@ -327,15 +383,17 @@ typedef struct editor_widget WEdit;
 #ifndef MIDNIGHT
 
 void edit_render_expose (WEdit * edit, XExposeEvent * xexpose);
+#ifndef GTK
 void edit_render_tidbits (struct cool_widget *w);
 int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent);
+#endif
 void edit_draw_menus (Window parent, int x, int y);
 void edit_run_make (void);
 void edit_change_directory (void);
 int edit_man_page_cmd (WEdit * edit);
 void edit_search_replace_dialog (Window parent, int x, int y, char **search_text, char **replace_text, char **arg_order, char *heading, int option);
 void edit_search_dialog (WEdit * edit, char **search_text);
-long edit_find (long search_start, unsigned char *exp, int *len, long last_byte, int (*get_byte) (void *, long), void *data);
+long edit_find (long search_start, unsigned char *expr, int *len, long last_byte, int (*get_byte) (void *, long), void *data);
 void edit_set_foreground_colors (unsigned long normal, unsigned long bold, unsigned long italic);
 void edit_set_background_colors (unsigned long normal, unsigned long abnormal, unsigned long marked, unsigned long marked_abnormal, unsigned long highlighted);
 void edit_set_cursor_color (unsigned long c);
@@ -360,7 +418,24 @@ int edit_translate_key (WEdit * edit, unsigned int x_keycode, long x_key, int x_
 
 #endif
 
+#ifndef NO_INLINE_GETBYTE
 int edit_get_byte (WEdit * edit, long byte_index);
+#else
+static inline int edit_get_byte (WEdit * edit, long byte_index)
+{
+    unsigned long p;
+    if (byte_index >= (edit->curs1 + edit->curs2) || byte_index < 0)
+	return '\n';
+
+    if (byte_index >= edit->curs1) {
+	p = edit->curs1 + edit->curs2 - byte_index - 1;
+	return edit->buffers2[p >> S_EDIT_BUF_SIZE][EDIT_BUF_SIZE - (p & M_EDIT_BUF_SIZE) - 1];
+    } else {
+	return edit->buffers1[byte_index >> S_EDIT_BUF_SIZE][byte_index & M_EDIT_BUF_SIZE];
+    }
+}
+#endif
+
 char *edit_get_buffer_as_text (WEdit * edit);
 int edit_load_file (WEdit * edit, const char *filename, const char *text, unsigned long text_size);
 int edit_count_lines (WEdit * edit, long current, int upto);
@@ -382,6 +457,8 @@ void edit_update_curs_col (WEdit * edit);
 void edit_block_copy_cmd (WEdit * edit);
 void edit_block_move_cmd (WEdit * edit);
 int edit_block_delete_cmd (WEdit * edit);
+int edit_block_delete (WEdit * edit);
+void edit_delete_line (WEdit * edit);
 
 int edit_delete (WEdit * edit);
 void edit_insert (WEdit * edit, int c);
@@ -444,6 +521,8 @@ void edit_split_filename (WEdit * edit, char *name);
 
 #ifdef MIDNIGHT
 #define CWidget Widget
+#elif defined(GTK)
+#define CWidget GtkEdit
 #endif
 void edit_set_syntax_change_callback (void (*callback) (CWidget *));
 void edit_load_syntax (WEdit * edit, char **names, char *type);
@@ -473,8 +552,8 @@ void edit_get_syntax_color (WEdit * edit, long byte_index, int *fg, int *bg);
 #    define mkdir(s,m) mc_mkdir(s,m)
 #    define itoa MY_itoa
 
-#    define edit_get_load_file(d,f,h) input_dialog (h, _(" Enter file name: "), f)
-#    define edit_get_save_file(d,f,h) input_dialog (h, _(" Enter file name: "), f)
+#    define edit_get_load_file(d,f,h) input_dialog (h, " Enter file name: ", f)
+#    define edit_get_save_file(d,f,h) input_dialog (h, " Enter file name: ", f)
 #    define CMalloc(x) malloc(x)
      
 #    define set_error_msg(s) edit_init_error_msg = strdup(s)
@@ -500,16 +579,140 @@ extern char *edit_init_error_msg;
 
 #else				/* ! MIDNIGHT */
 
+#    ifdef GTK
+#        define get_sys_error(s) (s)
+
+#        define open mc_open
+#        define close(f) mc_close(f)
+#        define read(f,b,c) mc_read(f,b,c)
+#        define write(f,b,c) mc_write(f,b,c)
+#        define stat(f,s) mc_stat(f,s)
+#        define mkdir(s,m) mc_mkdir(s,m)
+
+#        define itoa MY_itoa
+#        define CMalloc(x) malloc(x)
+
+#        define EDITOR_NO_FILE			(1<<3)
+#        define EDITOR_NO_SCROLL		(1<<4)
+#        define EDITOR_NO_TEXT			(1<<5)
+#        define EDITOR_HORIZ_SCROLL		(1<<6)
+
+#include <gdk/gdkprivate.h>
+#        define CWindowOf(w) (w)
+#        define CHeightOf(w) ((w)->editable.widget.allocation.height)
+#        define CWidthOf(w) ((w)->editable.widget.allocation.width)
+#        define COptionsOf(w) ((w)->options)
+
+#        define cache_type unsigned int
+
+/* font dimensions */
+#        define FONT_OVERHEAD		gtk_edit_option_text_line_spacing
+#        define FONT_BASE_LINE		(FONT_OVERHEAD + gtk_edit_option_font_ascent)
+#        define FONT_HEIGHT		(gtk_edit_option_font_ascent + gtk_edit_option_font_descent)
+#        define FONT_PIX_PER_LINE	(FONT_OVERHEAD + FONT_HEIGHT)
+#        define FONT_MEAN_WIDTH	gtk_edit_option_font_mean_width
+
+#        define FONT_OFFSET_X 0
+#        define FONT_OFFSET_Y		FONT_BASE_LINE
+
+#        define per_char gtk_edit_font_width_per_char
+
+#        ifndef _GTK_EDIT_C
+extern guchar gtk_edit_font_width_per_char[256];
+extern int gtk_edit_option_text_line_spacing;
+extern int gtk_edit_option_font_ascent;
+extern int gtk_edit_option_font_descent;
+extern int gtk_edit_option_font_mean_width;
+extern int gtk_edit_fixed_font;
+#        endif
+
+/* start temporary */
+
+#        define COLOR_BLACK 0
+#        define COLOR_WHITE 1
+#        define CURSOR_TYPE_EDITOR 0
+
+#        define WIN_MESSAGES GTK_WINDOW_TOPLEVEL, 20, 20
+#        define option_text_line_spacing 1
+#        define fixed_font 0
+
+#define color_palette(x) win->color[x].pixel
+
+#define DndNotDnd	-1
+#define DndUnknown	0
+#define DndRawData	1
+#define DndFile		2
+#define DndFiles	3
+#define DndText		4
+#define DndDir		5
+#define DndLink		6
+#define DndExe		7
+#define DndURL		8
+#define DndMIME         9
+
+#define DndEND		10
+
+#define dnd_null_term_type(d) \
+	((d) == DndFile || (d) == DndText || (d) == DndDir || \
+	(d) == DndLink || (d) == DndExe || (d) == DndURL)
+
+
+
+/* end temporary */
+
+#    else
+
+#        define WIN_MESSAGES edit->widget ? edit->widget->mainid : CRoot, 20, 20
+
+#    endif
+
 #    define MY_O_TEXT 0
-#    define WIN_MESSAGES edit->widget->mainid, 20, 20
-     
-#    define edit_get_load_file(d,f,h) CGetLoadFile(WIN_MESSAGES,d,f,h)
-#    define edit_get_save_file(d,f,h) CGetSaveFile(WIN_MESSAGES,d,f,h)
-#    define edit_error_dialog(h,t) CErrorDialog(WIN_MESSAGES,h,"%s",t)
-#    define edit_message_dialog(h,t) CMessageDialog(WIN_MESSAGES,0,h,"%s",t)
-#    define edit_query_dialog2(h,t,a,b) CQueryDialog(WIN_MESSAGES,h,t,a,b,0)
-#    define edit_query_dialog3(h,t,a,b,c) CQueryDialog(WIN_MESSAGES,h,t,a,b,c,0)
-#    define edit_query_dialog4(h,t,a,b,c,d) CQueryDialog(WIN_MESSAGES,h,t,a,b,c,d,0)
+
+#    ifdef GTK
+
+#        ifndef min
+#            define min(x,y)     (((x) < (y)) ? (x) : (y))
+#        endif
+
+#        ifndef max
+#            define max(x,y)     (((x) > (y)) ? (x) : (y))
+#        endif
+
+/*
+extern Display             *gdk_display;
+extern Window               gdk_root_window;
+*/
+
+enum {
+ match_file, match_normal
+};
+
+#        define edit_get_load_file(d,f,h) gtk_edit_dialog_get_load_file(d,f,h)
+#        define edit_get_save_file(d,f,h) gtk_edit_dialog_get_save_file(d,f,h)
+#        define edit_error_dialog(h,t) gtk_edit_dialog_error(h,"%s",t)
+#        define edit_message_dialog(h,t) gtk_edit_dialog_message(0,h,"%s",t)
+#        define edit_query_dialog2(h,t,a,b) gtk_edit_dialog_query(h,t,a,b,0)
+#        define edit_query_dialog3(h,t,a,b,c) gtk_edit_dialog_query(h,t,a,b,c,0)
+#        define edit_query_dialog4(h,t,a,b,c,d) gtk_edit_dialog_query(h,t,a,b,c,d,0)
+
+#        define CError(x) printf("Error: %s\n",x)
+#        define CIsDropAcknowledge(a,b) DndNotDnd
+#        define CGetDrop(e,d,s,x,y) DndNotDnd
+#        define CDropAcknowledge(x) 
+/* #        define edit_get_syntax_color(e,i,f,b)  */
+#        define get_international_character(k) 0
+#        define compose_key_pressed 0
+
+#    else
+
+#        define edit_get_load_file(d,f,h) CGetLoadFile(WIN_MESSAGES,d,f,h)
+#        define edit_get_save_file(d,f,h) CGetSaveFile(WIN_MESSAGES,d,f,h)
+#        define edit_error_dialog(h,t) CErrorDialog(WIN_MESSAGES,h,"%s",t)
+#        define edit_message_dialog(h,t) CMessageDialog(WIN_MESSAGES,0,h,"%s",t)
+#        define edit_query_dialog2(h,t,a,b) CQueryDialog(WIN_MESSAGES,h,t,a,b,0)
+#        define edit_query_dialog3(h,t,a,b,c) CQueryDialog(WIN_MESSAGES,h,t,a,b,c,0)
+#        define edit_query_dialog4(h,t,a,b,c,d) CQueryDialog(WIN_MESSAGES,h,t,a,b,c,d,0)
+#    endif
 
 #endif				/* ! MIDNIGHT */
 
@@ -517,6 +720,14 @@ extern char *home_dir;
 
 #define NUM_SELECTION_HISTORY 32
 
+#ifndef MAX_PATH_LEN
+#ifdef PATH_MAX
+#define MAX_PATH_LEN PATH_MAX
+#else
+#define MAX_PATH_LEN 1024
+#endif
+#endif
+
 #ifdef _EDIT_C
 
 struct selection selection =
@@ -557,7 +768,7 @@ int option_fake_half_tabs = 1;
 int option_save_mode = 0;
 int option_backup_ext_int = -1;
 int option_find_bracket = 1;
-int option_max_undo = 8192;
+int option_max_undo = 32768;
 
 int option_editor_fg_normal = 26;
 int option_editor_fg_bold = 8;
