@@ -134,7 +134,7 @@
 
 #include "popt.h"
 
-/* "$Id: main.c,v 1.48 1998/09/22 14:45:38 proskin Exp $" */
+/* "$Id: main.c,v 1.50 1998/10/05 18:07:26 norbert Exp $" */
 
 /* When the modes are active, left_panel, right_panel and tree_panel */
 /* Point to a proper data structure.  You should check with the functions */
@@ -907,13 +907,13 @@ directory_history_add (WPanel * panel, char *s)
 }
 
 /* Translate ftp://user:password@host/directory to 
-              #ftp:user:password@host/directory.
+              /#ftp:user:password@host/directory.
  */
 static char *
 translate_url_to_new_syntax (const char *p)
 {
     if (strncmp (p, "ftp://", 6) == 0)
-        return copy_strings ("#ftp:", p + 6, 0);
+        return copy_strings ("/#ftp:", p + 6, 0);
     else
         return strdup (p);
 }
@@ -922,6 +922,8 @@ translate_url_to_new_syntax (const char *p)
  *  If we moved to the parent directory move the selection pointer to
  *  the old directory name; If we leave VFS dir, remove FS specificator.
  *  Warn: This code spoils lwd string.
+ *
+ *  You do _NOT_ want to add any vfs aware code here. <pavel@ucw.cz>
  */
 char*
 get_parent_dir_name (char *cwd, char *lwd)
@@ -929,15 +931,7 @@ get_parent_dir_name (char *cwd, char *lwd)
     char *p;
     if (strlen (lwd) > strlen (cwd))
 	if((p=strrchr(lwd, PATH_SEP)) && !strncmp(cwd, lwd, p-lwd)){
-#ifdef USE_VFS
-	    char *q;
-	    if((q=strrchr(p, '#')) != 0){
-		/* Here we need proper VFS function */
-		if(!strcmp((q+1), "utar") || extfs_which(q+1) >= 0 || sfs_which(q+1) >= 0)
-		    *q = '\0';
-	    }
-#endif /* USE_VFS */
-	return (p+1);
+	    return (p+1);
 	}
     return NULL;
 }
