@@ -66,7 +66,7 @@ extern int ftpfs_use_passive_connections;
 extern int ftpfs_use_unix_list_options;
 #endif
 
-/* "$Id: setup.c,v 1.10 1998/09/15 19:41:23 norbert Exp $" */
+/* "$Id: setup.c,v 1.12 1998/10/11 19:19:06 norbert Exp $" */
 
 #ifdef USE_VFS
 extern int vfs_timeout;
@@ -226,7 +226,7 @@ static struct {
     { "ftpfs_retry_seconds", &ftpfs_retry_seconds },
     { "ftpfs_always_use_proxy", &ftpfs_always_use_proxy },
     { "ftpfs_use_passive_connections", &ftpfs_use_passive_connections },
-    { "ftpfs_use_unix_list_options", &ftpfs_use_passive_connections },
+    { "ftpfs_use_unix_list_options", &ftpfs_use_unix_list_options },
 #endif
 #endif
 #ifdef USE_INTERNAL_EDIT
@@ -415,7 +415,7 @@ void panel_load_setup (WPanel *panel, char *section)
 	}
 #ifndef PORT_HAS_ICON_VIEW
     if (panel->list_type == list_icons)
-        panel->list_type == list_full;
+        panel->list_type = list_full;
 #endif
     /* User formats */
     if (panel->user_format){
