@@ -78,7 +78,7 @@ int xtree_mode = 0;
 static int tree_callback (Dlg_head *h, WTree *tree, int msg, int par);
 #define tcallback (callback_fn) tree_callback
 
-/* "$Id: tree.c,v 1.7 1998/04/17 00:22:17 unammx Exp $" */
+/* "$Id: tree.c,v 1.9 1998/09/22 16:13:54 proskin Exp $" */
 
 /* Returns number of common characters */
 static inline int str_common (char *s1, char *s2)
@@ -752,11 +752,12 @@ void show_tree (WTree *tree)
 	if (current->sublevel == topsublevel){
 
 	    /* Top level directory */
-	    if (tree->active && current == tree->selected_ptr)
+	    if (tree->active && current == tree->selected_ptr) {
 		if (!use_colors && !tree->is_panel)
 			attrset (MARKED_COLOR);
 		else
 			attrset (SELECTED_COLOR);
+	    }
 
 	    /* Show full name */
 	    addstr (name_trunc (current->name, tree_cols - 6));
@@ -783,12 +784,13 @@ void show_tree (WTree *tree)
 	    addch (ACS_HLINE);
 	    noacs ();
 	    
-	    if (tree->active && current == tree->selected_ptr)
+	    if (tree->active && current == tree->selected_ptr) {
 		/* Selected directory -> change color */
 		if (!use_colors && !tree->is_panel)
 		    attrset (MARKED_COLOR);
 		else
 		    attrset (SELECTED_COLOR);
+	    }
 
 	    /* Show sub-name */
 	    addch (' ');
@@ -1528,7 +1530,7 @@ static int my_mkdir_rec (char *s, mode_t mode)
     /* FIXME: should check instead if s is at the root of that filesystem */
     if (!vfs_file_is_local (s))
 	return -1;
-    if (!strcmp (vfs_path(s), PATH_SEP_STR))
+    if (!strcmp (s, PATH_SEP_STR))
         return ENOTDIR;
     p = concat_dir_and_file (s, "..");
     q = vfs_canon (p);
