@@ -110,3 +110,15 @@ WMenu *menubar_new (int y, int x, int cols, Menu menu [], int items)
     return menubar;
 }
 
+void
+menubar_arrange (WMenu* menubar)
+{
+    /* nothing to do I think (Norbert) */
+}
+
+void
+destroy_menu (Menu menu)
+{
+    /* FIXME: need to implement? (Norbert) */
+}
+
