@@ -18,7 +18,7 @@
  * saves both computer's memory and my work.  <pavel@ucw.cz>
  * */
     
-static void *local_open (char *file, int flags, int mode)
+static void *local_open (vfs *me, char *file, int flags, int mode)
 {
     int *local_info;
     int fd;
@@ -64,12 +64,12 @@ int local_close (void *data)
     return close (fd);
 }
 
-int local_errno (void)
+int local_errno (vfs *me)
 {
     return errno;
 }
 
-static void *local_opendir (char *dirname)
+static void *local_opendir (vfs *me, char *dirname)
 {
     DIR **local_info;
     DIR *dir;
@@ -109,12 +109,12 @@ static int local_closedir (void *data)
     return i;
 }
 
-static int local_stat (char *path, struct stat *buf)
+static int local_stat (vfs *me, char *path, struct stat *buf)
 {
     return stat (path, buf);
 }
 
-static int local_lstat (char *path, struct stat *buf)
+static int local_lstat (vfs *me, char *path, struct stat *buf)
 {
 #ifndef HAVE_STATLSTAT
     return lstat (path,buf);
@@ -128,32 +128,32 @@ int local_fstat (void *data, struct stat *buf)
     return fstat (*((int *) data), buf);    
 }
 
-static int local_chmod (char *path, int mode)
+static int local_chmod (vfs *me, char *path, int mode)
 {
     return chmod (path, mode);
 }
 
-static int local_chown (char *path, int owner, int group)
+static int local_chown (vfs *me, char *path, int owner, int group)
 {
     return chown (path, owner, group);
 }
 
-static int local_utime (char *path, struct utimbuf *times)
+static int local_utime (vfs *me, char *path, struct utimbuf *times)
 {
     return utime (path, times);
 }
 
-static int local_readlink (char *path, char *buf, int size)
+static int local_readlink (vfs *me, char *path, char *buf, int size)
 {
     return readlink (path, buf, size);
 }
 
-static int local_unlink (char *path)
+static int local_unlink (vfs *me, char *path)
 {
     return unlink (path);
 }
 
-static int local_symlink (char *n1, char *n2)
+static int local_symlink (vfs *me, char *n1, char *n2)
 {
     return symlink (n1, n2);
 }
@@ -179,12 +179,12 @@ static int local_write (void *data, char *buf, int nbyte)
     return n;
 }
 
-static int local_rename (char *a, char *b)
+static int local_rename (vfs *me, char *a, char *b)
 {
     return rename (a, b);
 }
 
-static int local_chdir (char *path)
+static int local_chdir (vfs *me, char *path)
 {
     return chdir (path);
 }
@@ -196,27 +196,27 @@ int local_lseek (void *data, off_t offset, int whence)
     return lseek (fd, offset, whence);
 }
 
-static int local_mknod (char *path, int mode, int dev)
+static int local_mknod (vfs *me, char *path, int mode, int dev)
 {
     return mknod (path, mode, dev);
 }
 
-static int local_link (char *p1, char *p2)
+static int local_link (vfs *me, char *p1, char *p2)
 {
     return link (p1, p2);
 }
 
-static int local_mkdir (char *path, mode_t mode)
+static int local_mkdir (vfs *me, char *path, mode_t mode)
 {
     return mkdir (path, mode);
 }
 
-static int local_rmdir (char *path)
+static int local_rmdir (vfs *me, char *path)
 {
     return rmdir (path);
 }
 
-static vfsid local_getid (char *path, struct vfs_stamping **parent)
+static vfsid local_getid (vfs *me, char *path, struct vfs_stamping **parent)
 {
     *parent = NULL;
     return (vfsid) -1; /* We do not free local fs stuff at all */
@@ -231,30 +231,46 @@ static void local_free (vfsid id)
 {
 }
 
-static char *local_getlocalcopy (char *path)
+static char *local_getlocalcopy (vfs *me, char *path)
 {
     return strdup (path);
 }
 
-static void local_ungetlocalcopy (char *path, char *local, int has_changed)
+static void local_ungetlocalcopy (vfs *me, char *path, char *local, int has_changed)
 {
 }
 
 #ifdef HAVE_MMAP
-static caddr_t local_mmap (caddr_t addr, size_t len, int prot, int flags, void *data, off_t offset)
+static caddr_t local_mmap (vfs *me, caddr_t addr, size_t len, int prot, int flags, void *data, off_t offset)
 {
     int fd = * (int *)data;
 
     return mmap (addr, len, prot, flags, fd, offset);
 }
 
-static int local_munmap (caddr_t addr, size_t len, void *data)
+static int local_munmap (vfs *me, caddr_t addr, size_t len, void *data)
 {
     return munmap (addr, len);
 }
 #endif
 
+static int local_which(vfs *me, char *path)
+{
+    return 0;		/* Every path which other systems do not like is expected to be ours */
+}
+
 vfs local_vfs_ops = {
+    NULL,	/* This is place of next pointer */
+    "Local filesystem",
+    0,		/* flags */
+    NULL,	/* prefix */
+    NULL,	/* data */
+    0,		/* errno */
+    NULL,
+    NULL,
+    NULL,
+    local_which,
+
     local_open,
     local_close,
     local_read,
@@ -295,7 +311,6 @@ vfs local_vfs_ops = {
     local_mkdir,
     local_rmdir,
     
-    NULL,
     NULL,
     NULL
 #ifdef HAVE_MMAP
